/** **************************************************************************
 * policy.c
 *
 * Copyright 2008 Bryan Ischo <bryan@ischo.com>
 * (Actually this is added by nick huang <nickhuang99@hotmail.com> by following
 * coding style of Mr. Bryan Ishcho for new AWS S3 api.)
 *
 * This file is part of libs3.
 *
 * libs3 is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, version 3 of the License.
 *
 * In addition, as a special exception, the copyright holders give
 * permission to link the code of this library and its programs with the
 * OpenSSL library, and distribute linked combinations including the two.
 *
 * libs3 is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License version 3
 * along with libs3, in a file named COPYING.  If not, see
 * <http://www.gnu.org/licenses/>.
 *
 ************************************************************************** **/


#include <stdlib.h>
#include <string.h>
#include "libs3.h"
#include "request.h"

// Use a rather arbitrary max size for the document of 64K
#define BUCKET_POLICY_XML_DOC_MAXSIZE (64 * 1024)


/////////////////////////////////////////
// added by nick
typedef struct SetBucketPolicyData
{
    S3ResponsePropertiesCallback *responsePropertiesCallback;
    S3ResponseCompleteCallback *responseCompleteCallback;
    void *callbackData;

    int policyXmlDocumentLen;
    char policyXmlDocument[BUCKET_POLICY_XML_DOC_MAXSIZE];
    int policyXmlDocumentBytesWritten;

} SetBucketPolicyData;

//////////////////////////
// added by nick
typedef struct DeleteBucketPolicyData
{
    S3ResponsePropertiesCallback *responsePropertiesCallback;
    S3ResponseCompleteCallback *responseCompleteCallback;
    void *callbackData;
} DeleteBucketPolicyData;

///////////////////////////
// added by nick
static void setBucketPolicyCompleteCallback(S3Status requestStatus,
                                   const S3ErrorDetails *s3ErrorDetails,
                                   void *callbackData)
{
    SetBucketPolicyData *paData = (SetBucketPolicyData *) callbackData;

    (*(paData->responseCompleteCallback))
        (requestStatus, s3ErrorDetails, paData->callbackData);

    free(paData);
}

///////////////////////////////////
// added by nick
static int setBucketPolicyDataCallback(int bufferSize, char *buffer, void *callbackData)
{
    SetBucketPolicyData *paData = (SetBucketPolicyData *) callbackData;

    int remaining = (paData->policyXmlDocumentLen -
                     paData->policyXmlDocumentBytesWritten);

    int toCopy = bufferSize > remaining ? remaining : bufferSize;

    if (!toCopy) {
        return 0;
    }

    memcpy(buffer, &(paData->policyXmlDocument
                     [paData->policyXmlDocumentBytesWritten]), toCopy);

    paData->policyXmlDocumentBytesWritten += toCopy;

    return toCopy;
}
////////////////////////////
// added by nick
static S3Status setBucketPolicyPropertiesCallback
    (const S3ResponseProperties *responseProperties, void *callbackData)
{
    SetBucketPolicyData *paData = (SetBucketPolicyData *) callbackData;

    return (*(paData->responsePropertiesCallback))
        (responseProperties, paData->callbackData);
}

/////////////////////////////////////////
// added by nick
static void deleteBucketPolicyCompleteCallback(S3Status requestStatus,
                                         const S3ErrorDetails *s3ErrorDetails,
                                         void *callbackData)
{
    DeleteBucketPolicyData *dbData = (DeleteBucketPolicyData *) callbackData;

    (*(dbData->responseCompleteCallback))
        (requestStatus, s3ErrorDetails, dbData->callbackData);

    free(dbData);
}

////////////////////////////////////////////
// added by nick
static S3Status deleteBucketPolicyPropertiesCallback
    (const S3ResponseProperties *responseProperties, void *callbackData)
{
    DeleteBucketPolicyData *dbData = (DeleteBucketPolicyData *) callbackData;

    return (*(dbData->responsePropertiesCallback))
        (responseProperties, dbData->callbackData);
}
////////////////////////////////////////////////
// added by nick
void S3_get_bucket_policy(const S3BucketContext *bucketContext,
                   const S3GetConditions *getConditions,
                   uint64_t startByte, uint64_t byteCount,
                   S3RequestContext *requestContext,
                   const S3GetBucketPolicyHandler *handler, void *callbackData)
{
    // Set up the RequestParams
    RequestParams params =
    {
        HttpRequestTypeGET,                           // httpRequestType
        { bucketContext->bucketName,                  // bucketName
          bucketContext->protocol,                    // protocol
          bucketContext->uriStyle,                    // uriStyle
          bucketContext->accessKeyId,                 // accessKeyId
          bucketContext->secretAccessKey },           // secretAccessKey
        0,                                            // key
        0,                                            // queryParams
        "policy",                                     // subResource
        0,                                            // copySourceBucketName
        0,                                            // copySourceKey
        getConditions,                                // getConditions
        startByte,                                    // startByte
        byteCount,                                    // byteCount
        0,                                            // putProperties
        handler->responseHandler.propertiesCallback,  // propertiesCallback
        0,                                            // toS3Callback
        0,                                            // toS3CallbackTotalSize
        handler->getBucketPolicyDataCallback,         // fromS3Callback
        handler->responseHandler.completeCallback,    // completeCallback
        callbackData                                  // callbackData
    };

    // Perform the request
    request_perform(&params, requestContext);
}

//////////////////////////////////////////
// added by nick
void S3_delete_bucket_policy(S3Protocol protocol, S3UriStyle uriStyle,
                      const char *accessKeyId, const char *secretAccessKey,
                      const char *bucketName,
                      S3RequestContext *requestContext,
                      const S3ResponseHandler *handler, void *callbackData)
{
    // Create the callback data
    DeleteBucketPolicyData *dbData =
        (DeleteBucketPolicyData *) malloc(sizeof(DeleteBucketPolicyData));
    if (!dbData) {
        (*(handler->completeCallback))(S3StatusOutOfMemory, 0, callbackData);
        return;
    }

    dbData->responsePropertiesCallback = handler->propertiesCallback;
    dbData->responseCompleteCallback = handler->completeCallback;
    dbData->callbackData = callbackData;

    // Set up the RequestParams
    RequestParams params =
    {
        HttpRequestTypeDELETE,                        // httpRequestType
        { bucketName,                                 // bucketName
          protocol,                                   // protocol
          uriStyle,                                   // uriStyle
          accessKeyId,                                // accessKeyId
          secretAccessKey },                          // secretAccessKey
        0,                                            // key
        0,                                            // queryParams
        "policy",                                     // subResource
        0,                                            // copySourceBucketName
        0,                                            // copySourceKey
        0,                                            // getConditions
        0,                                            // startByte
        0,                                            // byteCount
        0,                                            // putProperties
        &deleteBucketPolicyPropertiesCallback,        // propertiesCallback
        0,                                            // toS3Callback
        0,                                            // toS3CallbackTotalSize
        0,                                            // fromS3Callback
        &deleteBucketPolicyCompleteCallback,                // completeCallback
        dbData                                        // callbackData
    };

    // Perform the request
    request_perform(&params, requestContext);
}

////////////////////////////////////////////
// added by nic
void S3_set_bucket_policy(const S3BucketContext *bucketContext, const char* policyBuf, int policyBufSize,
                S3RequestContext *requestContext,
                const S3ResponseHandler *handler, void *callbackData)
{

    SetBucketPolicyData *data = (SetBucketPolicyData *) malloc(sizeof(SetBucketPolicyData));
    if (!data) {
        (*(handler->completeCallback))(S3StatusOutOfMemory, 0, callbackData);
        return;
    }

    data->responsePropertiesCallback = handler->propertiesCallback;
    data->responseCompleteCallback = handler->completeCallback;
    data->callbackData = callbackData;

    data->policyXmlDocumentBytesWritten = 0;
    strncpy(data->policyXmlDocument, policyBuf, BUCKET_POLICY_XML_DOC_MAXSIZE);
    data->policyXmlDocumentLen = policyBufSize;

    // Set up the RequestParams
    RequestParams params =
    {
        HttpRequestTypePUT,                           // httpRequestType
        { bucketContext->bucketName,                  // bucketName
          bucketContext->protocol,                    // protocol
          bucketContext->uriStyle,                    // uriStyle
          bucketContext->accessKeyId,                 // accessKeyId
          bucketContext->secretAccessKey },           // secretAccessKey
        0,                                            // key
        0,                                            // queryParams
        "policy",                                     // subResource
        0,                                            // copySourceBucketName
        0,                                            // copySourceKey
        0,                                            // getConditions
        0,                                            // startByte
        0,                                            // byteCount
        0,                                            // putProperties
        &setBucketPolicyPropertiesCallback,           // propertiesCallback
        &setBucketPolicyDataCallback,                 // toS3Callback
        data->policyXmlDocumentLen,                   // toS3CallbackTotalSize
        0,                                            // fromS3Callback
        &setBucketPolicyCompleteCallback,             // completeCallback
        data                                          // callbackData
    };

    // Perform the request
    request_perform(&params, requestContext);
}

