/** **************************************************************************
 * website.c
 *
 * Copyright 2008 Bryan Ischo <bryan@ischo.com>
 * (Actually this is added by nick huang <nickhuang99@hotmail.com> by following
 * coding style of Mr. Bryan Ishcho for new AWS S3 api.)
 *
 * This file is part of libs3.
 *
 * libs3 is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation, version 3 of the License.
 *
 * In addition, as a special exception, the copyright holders give
 * permission to link the code of this library and its programs with the
 * OpenSSL library, and distribute linked combinations including the two.
 *
 * libs3 is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License version 3
 * along with libs3, in a file named COPYING.  If not, see
 * <http://www.gnu.org/licenses/>.
 *
 ************************************************************************** **/


#include <stdlib.h>
#include <string.h>
#include "libs3.h"
#include "request.h"

// Use a rather arbitrary max size for the document of 2K
#define BUCKET_WEBSITE_XML_DOC_MAXSIZE (2 * 1024)


typedef struct GetBucketWebsiteData
{
    SimpleXml simpleXml;
    S3ResponsePropertiesCallback *responsePropertiesCallback;
    S3ResponseCompleteCallback *responseCompleteCallback;
    void *callbackData;
    char *indexFile;
    char *errorFile;
    string_buffer(bucketWebsiteXmlDocument, BUCKET_WEBSITE_XML_DOC_MAXSIZE);
} GetBucketWebsiteData;


static S3Status getBucketWebsitePropertiesCallback(const S3ResponseProperties *responseProperties, void *callbackData)
{
    GetBucketWebsiteData *gaData = (GetBucketWebsiteData *) callbackData;

    return (*(gaData->responsePropertiesCallback))(responseProperties, gaData->callbackData);
}


static S3Status getBucketWebsiteDataCallback(int bufferSize, const char *buffer, void *callbackData)
{
    GetBucketWebsiteData *gaData = (GetBucketWebsiteData *) callbackData;

    int fit;

    string_buffer_append(gaData->bucketWebsiteXmlDocument, buffer, bufferSize, fit);

    return fit ? S3StatusOK : S3StatusXmlDocumentTooLarge;
}


static void getBucketWebsiteCompleteCallback(S3Status requestStatus,
                                   const S3ErrorDetails *s3ErrorDetails,
                                   void *callbackData)
{
    GetBucketWebsiteData *gaData = (GetBucketWebsiteData *) callbackData;

    if (requestStatus == S3StatusOK)
    {
        // Parse the document
        requestStatus = S3_convert_bucket_website(gaData->bucketWebsiteXmlDocument, gaData->indexFile, gaData->errorFile);
    }

    (*(gaData->responseCompleteCallback))(requestStatus, s3ErrorDetails, gaData->callbackData);

    free(gaData);
}

void S3_get_bucket_website(const S3BucketContext *bucketContext,
                char *indexFile, char *errorFile,
                S3RequestContext *requestContext,
                const S3ResponseHandler *handler, void *callbackData)
{
    // Create the callback data
    GetBucketWebsiteData *gaData = (GetBucketWebsiteData *) malloc(sizeof(GetBucketWebsiteData));
    if (!gaData)
    {
        (*(handler->completeCallback))(S3StatusOutOfMemory, 0, callbackData);
        return;
    }

    gaData->responsePropertiesCallback = handler->propertiesCallback;
    gaData->responseCompleteCallback = handler->completeCallback;
    gaData->callbackData = callbackData;

    gaData->indexFile = indexFile;
    gaData->errorFile = errorFile;
    string_buffer_initialize(gaData->bucketWebsiteXmlDocument);
    // Set up the RequestParams
    RequestParams params =
    {
        HttpRequestTypeGET,                           // httpRequestType
        { bucketContext->bucketName,                  // bucketName
          bucketContext->protocol,                    // protocol
          bucketContext->uriStyle,                    // uriStyle
          bucketContext->accessKeyId,                 // accessKeyId
          bucketContext->secretAccessKey },           // secretAccessKey
        0,                                            // key
        0,                                            // queryParams
        "website",                                    // subResource
        0,                                            // copySourceBucketName
        0,                                            // copySourceKey
        0,                                            // getConditions
        0,                                            // startByte
        0,                                            // byteCount
        0,                                            // putProperties
        &getBucketWebsitePropertiesCallback,                    // propertiesCallback
        0,                                            // toS3Callback
        0,                                            // toS3CallbackTotalSize
        &getBucketWebsiteDataCallback,                          // fromS3Callback
        &getBucketWebsiteCompleteCallback,                      // completeCallback
        gaData                                        // callbackData
    };
    // Perform the request
    request_perform(&params, requestContext);
}

// set website -------------------------------------------------------------------
/*********************************************************************************
<?xml version="1.0" encoding="UTF-8"?>
<WebsiteConfiguration xmlns="http://s3.amazonaws.com/doc/2006-03-01/">
  <IndexDocument>
    <Suffix>index.html</Suffix>
  </IndexDocument>
  <ErrorDocument>
    <Key>404.html</Key>
  </ErrorDocument>
</WebsiteConfiguration>
**********************************************************************************/

static S3Status generateBucketWebsiteXmlDocument(const char *indexFileName,
                                       const char *errorFileName,
                                       int *xmlDocumentLenReturn,
                                       char *xmlDocument,
                                       int xmlDocumentBufferSize)
{
    *xmlDocumentLenReturn = 0; //safe ??
    const char* STR_FORMAT =
    "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\
        <WebsiteConfiguration xmlns=\"http://s3.amazonaws.com/doc/2006-03-01/\">\
            <IndexDocument>\
                <Suffix>%s</Suffix>\
                    </IndexDocument>\
            <ErrorDocument>\
                <Key>%s</Key>\
            </ErrorDocument>\
        </WebsiteConfiguration>";

    *xmlDocumentLenReturn = snprintf(xmlDocument, xmlDocumentBufferSize,
        STR_FORMAT, indexFileName, errorFileName);
    if (*xmlDocumentLenReturn >= xmlDocumentBufferSize || *xmlDocumentLenReturn <= 0)
    {
        return S3StatusXmlDocumentTooLarge;
    }
    return S3StatusOK;
}

typedef struct SetBucketWebsiteData
{
    S3ResponsePropertiesCallback *responsePropertiesCallback;
    S3ResponseCompleteCallback *responseCompleteCallback;
    void *callbackData;

    int bucketWebsiteXmlDocumentLen;
    char bucketWebsiteXmlDocument[BUCKET_WEBSITE_XML_DOC_MAXSIZE];
    int bucketWebsiteXmlDocumentBytesWritten;

} SetBucketWebsiteData;


static S3Status setBucketWebsitePropertiesCallback(const S3ResponseProperties *responseProperties, void *callbackData)
{
    SetBucketWebsiteData *paData = (SetBucketWebsiteData *) callbackData;

    return (*(paData->responsePropertiesCallback))(responseProperties, paData->callbackData);
}

static int setBucketWebsiteDataCallback(int bufferSize, char *buffer, void *callbackData)
{
    SetBucketWebsiteData *paData = (SetBucketWebsiteData *) callbackData;

    int remaining = (paData->bucketWebsiteXmlDocumentLen -
                     paData->bucketWebsiteXmlDocumentBytesWritten);

    int toCopy = bufferSize > remaining ? remaining : bufferSize;

    if (!toCopy)
    {
        return 0;
    }

    memcpy(buffer, &(paData->bucketWebsiteXmlDocument[paData->bucketWebsiteXmlDocumentBytesWritten]), toCopy);
    paData->bucketWebsiteXmlDocumentBytesWritten += toCopy;
    return toCopy;
}

static void setBucketWebsiteCompleteCallback(S3Status requestStatus,
                                   const S3ErrorDetails *s3ErrorDetails,
                                   void *callbackData)
{
    SetBucketWebsiteData *paData = (SetBucketWebsiteData *) callbackData;

    (*(paData->responseCompleteCallback))
        (requestStatus, s3ErrorDetails, paData->callbackData);

    free(paData);
}


void S3_set_bucket_website(const S3BucketContext *bucketContext,
                const char *indexFileName,
                const char *errorFileName,
                S3RequestContext *requestContext,
                const S3ResponseHandler *handler, void *callbackData)
{
    SetBucketWebsiteData *data = (SetBucketWebsiteData *) malloc(sizeof(SetBucketWebsiteData));
    if (!data)
    {
        (*(handler->completeCallback))(S3StatusOutOfMemory, 0, callbackData);
        return;
    }
    // Convert aclGrants to XML document
    S3Status status = generateBucketWebsiteXmlDocument
        (indexFileName, errorFileName,
         &(data->bucketWebsiteXmlDocumentLen), data->bucketWebsiteXmlDocument,
         sizeof(data->bucketWebsiteXmlDocument));
    if (status != S3StatusOK)
    {
        free(data);
        (*(handler->completeCallback))(status, 0, callbackData);
        return;
    }

    data->responsePropertiesCallback = handler->propertiesCallback;
    data->responseCompleteCallback = handler->completeCallback;
    data->callbackData = callbackData;

    data->bucketWebsiteXmlDocumentBytesWritten = 0;

    // Set up the RequestParams
    RequestParams params =
    {
        HttpRequestTypePUT,                           // httpRequestType
        { bucketContext->bucketName,                  // bucketName
          bucketContext->protocol,                    // protocol
          bucketContext->uriStyle,                    // uriStyle
          bucketContext->accessKeyId,                 // accessKeyId
          bucketContext->secretAccessKey },           // secretAccessKey
        0,                                            // key
        0,                                            // queryParams
        "website",                                    // subResource
        0,                                            // copySourceBucketName
        0,                                            // copySourceKey
        0,                                            // getConditions
        0,                                            // startByte
        0,                                            // byteCount
        0,                                            // putProperties
        &setBucketWebsitePropertiesCallback,                // propertiesCallback
        &setBucketWebsiteDataCallback,                      // toS3Callback
        data->bucketWebsiteXmlDocumentLen,                  // toS3CallbackTotalSize
        0,                                            // fromS3Callback
        &setBucketWebsiteCompleteCallback,                  // completeCallback
        data                                          // callbackData
    };

    // Perform the request
    request_perform(&params, requestContext);
}


// added by nick
typedef struct DeleteBucketWebsiteData
{
    S3ResponsePropertiesCallback *responsePropertiesCallback;
    S3ResponseCompleteCallback *responseCompleteCallback;
    void *callbackData;
} DeleteBucketWebsiteData;


static void deleteBucketWebsiteCompleteCallback(S3Status requestStatus,
                                         const S3ErrorDetails *s3ErrorDetails,
                                         void *callbackData)
{
    DeleteBucketWebsiteData *dbData = (DeleteBucketWebsiteData *) callbackData;

    (*(dbData->responseCompleteCallback))
        (requestStatus, s3ErrorDetails, dbData->callbackData);

    free(dbData);
}

////////////////////////////////////////////
// added by nick
static S3Status deleteBucketWebsitePropertiesCallback
    (const S3ResponseProperties *responseProperties, void *callbackData)
{
    DeleteBucketWebsiteData *dbData = (DeleteBucketWebsiteData *) callbackData;

    return (*(dbData->responsePropertiesCallback))
        (responseProperties, dbData->callbackData);
}

void S3_delete_bucket_website(S3Protocol protocol, S3UriStyle uriStyle,
                      const char *accessKeyId, const char *secretAccessKey,
                      const char *bucketName,
                      S3RequestContext *requestContext,
                      const S3ResponseHandler *handler, void *callbackData)
{
    // Create the callback data
    DeleteBucketWebsiteData *dbData =
        (DeleteBucketWebsiteData *) malloc(sizeof(DeleteBucketWebsiteData));
    if (!dbData)
    {
        (*(handler->completeCallback))(S3StatusOutOfMemory, 0, callbackData);
        return;
    }

    dbData->responsePropertiesCallback = handler->propertiesCallback;
    dbData->responseCompleteCallback = handler->completeCallback;
    dbData->callbackData = callbackData;

    // Set up the RequestParams
    RequestParams params =
    {
        HttpRequestTypeDELETE,                        // httpRequestType
        { bucketName,                                 // bucketName
          protocol,                                   // protocol
          uriStyle,                                   // uriStyle
          accessKeyId,                                // accessKeyId
          secretAccessKey },                          // secretAccessKey
        0,                                            // key
        0,                                            // queryParams
        "website",                                     // subResource
        0,                                            // copySourceBucketName
        0,                                            // copySourceKey
        0,                                            // getConditions
        0,                                            // startByte
        0,                                            // byteCount
        0,                                            // putProperties
        &deleteBucketWebsitePropertiesCallback,        // propertiesCallback
        0,                                            // toS3Callback
        0,                                            // toS3CallbackTotalSize
        0,                                            // fromS3Callback
        &deleteBucketWebsiteCompleteCallback,                // completeCallback
        dbData                                        // callbackData
    };

    // Perform the request
    request_perform(&params, requestContext);
}

